@@ -50,8 +50,6 @@
 
 #define ENABLE_HELPER  1
 
-//#define DEBUG_IM  1
-
 
 const short kMENU_Pencil = kBaseResourceID + 1;
 
@@ -86,6 +84,11 @@ enum
 MUIMSessionHandle gActiveSession;
 Boolean gDisableFocusedContext = FALSE;
 
+int gNumSession = 0;
+MUIMSessionHandle *gSessionList = NULL;
+
+uim_context gUC = NULL;
+
 int gUimFD;
 CFSocketRef gUimSock;
 CFRunLoopSourceRef gUimRun;
@@ -243,6 +246,28 @@ MUIMInitialize(ComponentInstance inComponentInstance,
   gUimSock = NULL;
   gUimRun = NULL;
 
+  {
+    const char *imName;
+    CFPropertyListRef imVal;
+    CFStringRef appID = CFSTR(kAppID);
+
+    // Load the preferences
+    imVal = CFPreferencesCopyAppValue(CFSTR("InputMethod"), appID);
+    if (imVal && CFGetTypeID(imVal) == CFStringGetTypeID())
+      imName = CFStringGetCStringPtr((CFStringRef) imVal, kCFStringEncodingASCII);
+    else
+      imName = kDefaultIM;
+
+    DEBUG_PRINT("MUIMSessionOpen() imName='%s'\n", imName);
+
+    gUC = uim_create_context(gSessionList, "UTF-8", NULL, imName,
+                             uim_iconv, UIMCommitString);
+    if (!gUC) {
+      DEBUG_PRINT("uim_create_context() failed\n");
+      result = memFullErr;
+    }
+  }
+
   return result;
 }
 
@@ -269,82 +294,54 @@ MUIMSessionOpen(ComponentInstance inComponentInstance,
                 MUIMSessionHandle *outSessionHandle)
 {
   ComponentResult result = noErr;
-  CFStringRef appID = CFSTR(kAppID);
-  CFPropertyListRef imVal;
-  const char *imName;
-
-#ifdef DEBUG_IM
-  UInt32 i;
-#endif
-
-  // Load the value
-  imVal = CFPreferencesCopyAppValue(CFSTR("InputMethod"), appID);
-  if (imVal && CFGetTypeID(imVal) == CFStringGetTypeID())
-    imName = CFStringGetCStringPtr((CFStringRef) imVal, kCFStringEncodingASCII);
-  else
-    imName = kDefaultIM;
-
-  DEBUG_PRINT("MUIMSessionOpen() imName='%s'\n", imName);
 
   if (*outSessionHandle == nil) {
     *outSessionHandle =
       (MUIMSessionHandle) NewHandle(sizeof(MUIMSessionRecord));
-    DEBUG_PRINT("MUIMSessionOpen() NewHandle=%p\n", *outSessionHandle);
+    //DEBUG_PRINT("MUIMSessionOpen() NewHandle=%p\n", *outSessionHandle);
   }
 
   if (*outSessionHandle) {
+    if (!gSessionList)
+      gSessionList = (MUIMSessionHandle *)
+        malloc(sizeof(MUIMSessionHandle) * gNumSession);
+    else
+      gSessionList = (MUIMSessionHandle *)
+        realloc(gSessionList, sizeof(MUIMSessionHandle) * gNumSession);
+
+    gSessionList[gNumSession] = *outSessionHandle;
+    gNumSession++;
+
     (**outSessionHandle)->fComponentInstance = inComponentInstance;
     (**outSessionHandle)->fLastUpdateLength = 0;
     (**outSessionHandle)->fSegments = NULL;
     (**outSessionHandle)->fSegmentCount = 0;
 
-    (**outSessionHandle)->fUC =
-      uim_create_context(*outSessionHandle, "UTF-8", NULL, imName,
-                         uim_iconv, UIMCommitString);
-
-    if (!((**outSessionHandle)->fUC))
-      (**outSessionHandle)->fNumIM = 0;
-    else
-      (**outSessionHandle)->fNumIM = uim_get_nr_im((**outSessionHandle)->fUC);
-
-    if ((**outSessionHandle)->fUC == NULL) {
-      DEBUG_PRINT("uim_create_context() failed\n");
-      result = memFullErr;
-    }
-
-#ifdef DEBUG_IM
-    for (i = 0; i < (**outSessionHandle)->fNumIM; i++) {
-      DEBUG_PRINT("MUIMSessionOpen() IM %lu: '%s' '%s'\n",
-                  uim_get_im_name((**outSessionHandle)->fUC, i),
-                  uim_get_im_language((**outSessionHandle)->fUC, i));
-    }
-#endif
+    //(**outSessionHandle)->fUC = gUC;
 
 #ifdef ENABLE_HELPER
     UIMCheckHelper(*outSessionHandle);
 #endif
 
-    uim_set_preedit_cb((**outSessionHandle)->fUC,
+    uim_set_preedit_cb(gUC,
                        UIMPreeditClear,
                        UIMPreeditPushback,
                        UIMPreeditUpdate);
 #ifdef ENABLE_HELPER
-    uim_set_prop_list_update_cb((**outSessionHandle)->fUC,
+    uim_set_prop_list_update_cb(gUC,
                                 UIMUpdatePropList);
-    uim_set_prop_label_update_cb((**outSessionHandle)->fUC,
+    uim_set_prop_label_update_cb(gUC,
                                  UIMUpdatePropLabel);
 
-    uim_prop_list_update((**outSessionHandle)->fUC);
+    uim_prop_list_update(gUC);
+    uim_prop_label_update(gUC);
 #endif
-    uim_set_candidate_selector_cb((**outSessionHandle)->fUC,
+    uim_set_candidate_selector_cb(gUC,
                                   UIMCandAcivate,
                                   UIMCandSelect,
                                   UIMCandShiftPage,
                                   UIMCandDeactivate);
 
-    (**outSessionHandle)->fMode =
-      uim_get_current_mode((**outSessionHandle)->fUC);
-
     (**outSessionHandle)->fLastUpdateLength = 0;
     (**outSessionHandle)->fFixBuffer = NULL;
     (**outSessionHandle)->fFixLen = 0;
@@ -358,17 +355,36 @@ MUIMSessionOpen(ComponentInstance inComponentInstance,
   else
     result = memFullErr;
 
+  /*
+  // press Kana-key to enable label
+  uim_press_key((**outSessionHandle)->fUC, UKey_Private2, 0);
+  uim_release_key((**outSessionHandle)->fUC, UKey_Private2, 0);
+
+  // press Eisu-key
+  uim_press_key((**outSessionHandle)->fUC, UKey_Private1, 0);
+  uim_release_key((**outSessionHandle)->fUC, UKey_Private1, 0);
+  */
+
   return result;
 }
 
 void
 MUIMSessionClose(MUIMSessionHandle inSessionHandle)
 {
-  UInt32 i;
+  UInt32 i, j;
 
   DEBUG_PRINT("MUIMSessionClose() inSessionHandle=%p\n",
               inSessionHandle);
 
+  for (i = 0; i < gNumSession; i++) {
+    if (gSessionList[i] == inSessionHandle) {
+      gSessionList[i] = NULL;
+      for (j = i; j < gNumSession - 1; j++)
+        gSessionList[j] = gSessionList[j + 1];
+    }
+  }
+  gNumSession--;
+
   if (inSessionHandle) {
     for (i = 0; i < (*inSessionHandle)->fSegmentCount; i++)
       free((*inSessionHandle)->fSegments[i].fBuffer);
@@ -399,10 +415,10 @@ MUIMSessionActivate(MUIMSessionHandle inSessionHandle)
 #ifdef ENABLE_HELPER
   UIMCheckHelper(inSessionHandle);
 
-  uim_prop_list_update((*inSessionHandle)->fUC);
-  uim_prop_label_update((*inSessionHandle)->fUC);
+  uim_prop_list_update(gUC);
+  uim_prop_label_update(gUC);
 
-  uim_helper_client_focus_in((*inSessionHandle)->fUC);
+  uim_helper_client_focus_in(gUC);
 #endif
 
   return result;
@@ -415,7 +431,7 @@ MUIMSessionDeactivate(MUIMSessionHandle inSessionHandle)
               gActiveSession);
 
 #ifdef ENABLE_HELPER
-  uim_helper_client_focus_out((*inSessionHandle)->fUC);
+  uim_helper_client_focus_out(gUC);
 #endif
 
   /*
@@ -538,9 +554,9 @@ MUIMHandleInput(MUIMSessionHandle inSessionHandle, UInt32 inKeycode,
 
   DEBUG_PRINT("MUIMHandleInput() key=0x%x mod=0x%x\n", key, mod);
 
-  rv = uim_press_key((*inSessionHandle)->fUC, key, mod);
+  rv = uim_press_key(gUC, key, mod);
 
-  uim_release_key((*inSessionHandle)->fUC, key, mod);
+  uim_release_key(gUC, key, mod);
 
   if (!rv || isScriptKey)
     handled = TRUE;
