@@ -63,17 +63,23 @@ MUIMUpdateActiveInputArea(MUIMSessionHandle inHandle, Boolean inFix)
   UniCharPtr str = NULL;
   UInt32 len = 0, cursorPos = 0;
   SInt32 fixLen;
+  UInt32 i, segmentPosition = 0;
 
   if (!inFix) {
     GetPreeditString(inHandle, &str, &len, &cursorPos);
 
+    DEBUG_PRINT("MUIMUpdateActiveInputArea() [NoFix] fOldPreeditLen=%lu len=%lu cursorPos=%lu\n",
+                (*inHandle)->fOldPreeditLen, len, cursorPos);
+
     if ((*inHandle)->fOldPreedit != NULL &&
-        !memcmp((*inHandle)->fOldPreedit, str, sizeof(UniChar) * len)) {
+        //!memcmp((*inHandle)->fOldPreedit, str, sizeof(UniChar) * (len + 1))
+        (*inHandle)->fOldPreeditLen == 0 && len == 0) {
       free(str);
       return noErr;
     }
     free((*inHandle)->fOldPreedit);
     (*inHandle)->fOldPreedit = str;
+    (*inHandle)->fOldPreeditLen = len;
     fixLen = 0;
   }
   else {
@@ -174,18 +180,43 @@ MUIMUpdateActiveInputArea(MUIMSessionHandle inHandle, Boolean inFix)
   if (error == noErr) {
     hiliteRangePtr =
       (TextRangeArrayPtr) NewPtrClear(sizeof(short) +
-                                      sizeof(TextRange) * 2);
+                                      sizeof(TextRange) *
+                                      ((*inHandle)->fSegmentCount + 1));
     if (hiliteRangePtr) {
-      hiliteRangePtr->fNumOfRanges = 2;
-      hiliteRangePtr->fRange[0].fStart = 0;
-      hiliteRangePtr->fRange[0].fEnd = len * sizeof(UniChar);
-      if (inFix)
-        hiliteRangePtr->fRange[0].fHiliteStyle = kConvertedText;
-      else
-        hiliteRangePtr->fRange[0].fHiliteStyle = kRawText;
-      hiliteRangePtr->fRange[1].fStart = cursorPos * sizeof(UniChar);
-      hiliteRangePtr->fRange[1].fEnd = cursorPos * sizeof(UniChar);
-      hiliteRangePtr->fRange[1].fHiliteStyle = kCaretPosition;
+      hiliteRangePtr->fNumOfRanges = (*inHandle)->fSegmentCount + 1;
+      for (i = 0; i < (*inHandle)->fSegmentCount; i++) {
+        hiliteRangePtr->fRange[i].fStart = segmentPosition * sizeof(UniChar);
+        segmentPosition += (*inHandle)->fSegments[i].fLength;
+        hiliteRangePtr->fRange[i].fEnd = segmentPosition * sizeof(UniChar);
+        /*
+         * kCaretPosition = 1,
+         * kRawText = 2,
+         * kSelectedRawText = 3,
+         * kConvertedText = 4,
+         * kSelectedConvertedText = 5,
+         * kBlockFillText = 6,
+         * kOutlineText = 7,
+         * kSelectedText = 8
+         */
+        if ((*inHandle)->fSegments[i].fAttr & (UPreeditAttr_Reverse | UPreeditAttr_Cursor)) {
+          hiliteRangePtr->fRange[i].fHiliteStyle = kSelectedConvertedText;
+        } else if (((*inHandle)->fSegments[i].fLength == 0) ||
+                 ((*inHandle)->fSegments[i].fLength == len) ||
+                 ((i > 0) && hiliteRangePtr->fRange[i - 1].fHiliteStyle == kRawText) ||
+                 /* XXX hack aleart! */
+                 /* for Anthy */
+                 ((i == 0) && ((*inHandle)->fSegmentCount > 2) && ((len - (*inHandle)->fSegments[i].fLength) == 1)) ||
+                 ((i == 0) && ((*inHandle)->fSegmentCount > 2) && ((len - (*inHandle)->fSegments[i].fLength) == 2)) ||
+                 /* for SKK */
+                 ((i == 0) && ((*inHandle)->fSegmentCount == 4) && ((*inHandle)->fSegments[i].fLength == 1))) {
+          hiliteRangePtr->fRange[i].fHiliteStyle = kRawText;
+        } else {
+          hiliteRangePtr->fRange[i].fHiliteStyle = kConvertedText;
+        }
+      }
+      hiliteRangePtr->fRange[i].fStart = cursorPos * sizeof(UniChar);
+      hiliteRangePtr->fRange[i].fEnd = cursorPos * sizeof(UniChar);
+      hiliteRangePtr->fRange[i].fHiliteStyle = kCaretPosition;
     }
     else
       error = memFullErr;
@@ -193,7 +224,7 @@ MUIMUpdateActiveInputArea(MUIMSessionHandle inHandle, Boolean inFix)
   if (error == noErr)
     error = SetEventParameter(event, kEventParamTextInputSendHiliteRng,
                               typeTextRangeArray,
-                              sizeof(short) + sizeof(TextRange) * 2,
+                              sizeof(short) + sizeof(TextRange) * (i + 1),
                               hiliteRangePtr);
 
   //DEBUG_PRINT("MUIMUpdateActiveInputArea - fRange[0].fEnd=%d fRange[1].fStart=%d fRange[1].fEnd=%d\n",
