@@ -31,6 +31,7 @@
   SUCH DAMAGE.
 */
 
+#import "unistd.h"
 #import <uim/uim.h>
 #import "Preference.h"
 #import "MacUIMPrefPane.h"
@@ -70,8 +71,6 @@
 			  NULL, NULL, uim_iconv, NULL);
   if (uc) {
     numModules = uim_get_nr_im(uc);
-    printf("MacUIMPrefPane::initWithBundle() numModules=%d imButton=%p\n",
-           numModules, imButton);
     imModules = (IMModule **) malloc(sizeof(IMModule *) * numModules);
     if (*imModules != NULL) {
       int i;
@@ -81,10 +80,6 @@
           imModules[i]->index = i;
           imModules[i]->name = strdup(uim_get_im_name(uc, i));
           imModules[i]->lang = strdup(uim_get_im_language(uc, i));
-
-          printf("%s (%s)\n",
-                 imModules[i]->name,
-                 imModules[i]->lang);
         }
       }
     }
@@ -102,21 +97,6 @@
   // Load the value
   imVal = CFPreferencesCopyAppValue(CFSTR("InputMethod"), appID);
 
-#if 0
-  if (imVal && CFGetTypeID(imVal) == CFStringGetTypeID()) {
-    if (CFStringCompare((CFStringRef) imVal, CFSTR(kIMStrAnthy), NULL)
-        == kCFCompareEqualTo)
-      [imButton selectItemAtIndex:kIMTypeAnthy];
-    else if (CFStringCompare((CFStringRef) imVal, CFSTR(kIMStrPRIME), NULL)
-             == kCFCompareEqualTo)
-      [imButton selectItemAtIndex:kIMTypePRIME];
-    else
-      [imButton selectItemAtIndex:kIMTypeSKK];
-  }
-  else
-    [imButton selectItemAtIndex:kIMTypeSKK];
-#endif
-
   [imButton removeAllItems];
 
   for (i = 0; i < numModules; i++) {
@@ -139,10 +119,23 @@
       }
     }
   }
+  else {
+    for (i = 0; i < numModules; i++) {
+      if ([[NSString stringWithCString:kDefaultIM] isEqualToString:[NSString stringWithCString:imModules[i]->name]]) {
+        [imButton selectItemAtIndex:i];
+        break;
+      }
+    }
+  }
 
   [tab selectFirstTabViewItem:self];
 }
 
+- (void)willSelect
+{
+  [appletButton setState:([self isExtraLoaded:kHelperID] ? NSOnState : NSOffState)];
+}
+
 - (void)didUnselect
 {
   CFNotificationCenterRef center;
@@ -170,6 +163,25 @@
   CFPreferencesAppSynchronize(appID);
 }
 
+- (IBAction)appletChange:(id)sender
+{
+  // Extra load
+  if (([appletButton state] == NSOnState)
+      && ![self isExtraLoaded:kHelperID]) {
+    [self loadExtra];
+    if (![self isExtraLoaded:kHelperID]) {
+      [appletButton setState:NSOffState];
+    }
+  }
+  else if (([appletButton state] == NSOffState)
+           && [self isExtraLoaded:kHelperID]) {
+    [self removeExtra:kHelperID];
+    if ([self isExtraLoaded:kHelperID]) {
+      [appletButton setState:NSOnState];
+    }
+  }
+}
+
 - (void)prefSync
 {
   int i = [[imButton objectValue] intValue];
@@ -185,4 +197,67 @@
   }
 }
 
+- (void)loadExtra
+{
+  int sleepCount = 0;
+  NSURL *crackerUrl;
+  NSURL *helperUrl =
+    [NSURL fileURLWithPath:[[self bundle]
+                             pathForResource:@"MacUIMHelper"
+                             ofType:@"menu"
+                             inDirectory:@""]];
+
+  // Try default MenuCracker installed by user
+  CoreMenuExtraAddMenuExtra((CFURLRef) helperUrl,
+                            0, 0, 0, 0, 0);
+
+  while (![self isExtraLoaded:kHelperID]
+         && (sleepCount < 2000000)) {
+    sleepCount += 250000;
+    usleep(250000);
+  }
+
+  if (![self isExtraLoaded:kHelperID]) {
+    // No load
+    if (![self isExtraLoaded:kMenuCrackerBundleID]) {
+      // Load the cracker
+      crackerUrl = [NSURL fileURLWithPath:[[self bundle] pathForResource:@"MenuCracker"
+                                                         ofType:@"menu"
+                                                         inDirectory:@""]];
+      CoreMenuExtraAddMenuExtra((CFURLRef) crackerUrl,
+                                0, 0, 0, 0, 0);
+      // Load the request again
+      CoreMenuExtraAddMenuExtra((CFURLRef) helperUrl,
+                                0, 0, 0, 0, 0);
+    }
+  }
+
+  sleepCount = 0;
+  while (![self isExtraLoaded:kHelperID]
+         && (sleepCount < 2000000)) {
+    sleepCount += 250000;
+    usleep(250000);
+  }
+}
+
+- (void)removeExtra:(NSString *)extraID
+{
+  void *extra;
+
+  if ((CoreMenuExtraGetMenuExtra((CFStringRef)extraID, &extra) == 0)
+      && extra)
+    CoreMenuExtraRemoveMenuExtra(extra, 0);
+}
+
+- (BOOL)isExtraLoaded:(NSString *)extraID
+{
+  void *extra;
+
+  if (CoreMenuExtraGetMenuExtra((CFStringRef) extraID, &extra) == 0
+      && extra)
+    return YES;
+
+  return NO;
+}
+
 @end
