@@ -37,15 +37,15 @@
 #include <uim/uim.h>
 #include <uim/uim-helper.h>
 
+
+#define kPropListUpdate   "prop_list_update\ncharset=UTF-8"
+
+#define kPropLabelUpdate  "prop_label_update\ncharset=UTF-8"
+
+
 void
 UIMCommitString(void *ptr, const char *str);
 
-void
-UIMHelperDisconnect();
-
-void
-UIMMode(void *ptr, int mode);
-
 void
 UIMPreeditClear(void *ptr);
 
@@ -56,7 +56,7 @@ void
 UIMPreeditUpdate(void *ptr);
 
 void
-UIMCandAcivate(void *inPrt, int inNR, int inLimit);
+UIMCandAcivate(void *inPtr, int inNR, int inLimit);
 
 void
 UIMCandSelect(void *inPtr, int inIndex);
@@ -67,5 +67,20 @@ UIMCandShiftPage(void *inPtr, int inDirection);
 void
 UIMCandDeactivate(void *inPtr);
 
+void
+UIMUpdatePropList(void *inPtr, const char *inStr);
+
+void
+UIMUpdatePropLabel(void *inPtr, const char *inStr);
+
+void
+UIMCheckHelper(MUIMSessionHandle inHandle);
+
+void
+UIMHelperDisconnect();
+
+void
+UIMHelperClose();
+
 
 #endif  /* __UIM_CALLBACK_H__ */
