@@ -44,7 +44,7 @@ static char *levelmsg[] = {
 };
 
 #ifndef DEBUG
-static UInt32 sDebugLeve = ERROR_NORMAL;
+static UInt32 sDebugLeve = ERROR_WARNING;
 #else
 static UInt32 sDebugLeve = ERROR_DEBUG;
 #endif
